# -*- coding: utf-8 -*-

#
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_DialogOptions(object):
    def setupUi(self, DialogOptions):
        DialogOptions.setObjectName("DialogOptions")
        DialogOptions.resize(814, 822)
        icon = QtGui.QIcon.fromTheme("configure")
        DialogOptions.setWindowIcon(icon)
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(DialogOptions)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.tabWidget = QtWidgets.QTabWidget(DialogOptions)
        self.tabWidget.setObjectName("tabWidget")
        self.tabGeneration = QtWidgets.QWidget()
        self.tabGeneration.setObjectName("tabGeneration")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout(self.tabGeneration)
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.gridLayout_4 = QtWidgets.QGridLayout()
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.labelFuzzingSamples = QtWidgets.QLabel(self.tabGeneration)
        self.labelFuzzingSamples.setObjectName("labelFuzzingSamples")
        self.gridLayout_4.addWidget(self.labelFuzzingSamples, 1, 1, 1, 1)
        self.checkBoxDefaultFuzzingPause = QtWidgets.QCheckBox(self.tabGeneration)
        self.checkBoxDefaultFuzzingPause.setObjectName("checkBoxDefaultFuzzingPause")
        self.gridLayout_4.addWidget(self.checkBoxDefaultFuzzingPause, 0, 0, 1, 2)
        self.doubleSpinBoxFuzzingPause = KillerDoubleSpinBox(self.tabGeneration)
        self.doubleSpinBoxFuzzingPause.setDecimals(3)
        self.doubleSpinBoxFuzzingPause.setMaximum(999999999.0)
        self.doubleSpinBoxFuzzingPause.setObjectName("doubleSpinBoxFuzzingPause")
        self.gridLayout_4.addWidget(self.doubleSpinBoxFuzzingPause, 1, 0, 1, 1)
        self.checkBoxMultipleModulations = QtWidgets.QCheckBox(self.tabGeneration)
        self.checkBoxMultipleModulations.setObjectName("checkBoxMultipleModulations")
        self.gridLayout_4.addWidget(self.checkBoxMultipleModulations, 2, 0, 1, 2)
        self.verticalLayout_9.addLayout(self.gridLayout_4)
        self.groupBoxModulationAccuracy = QtWidgets.QGroupBox(self.tabGeneration)
        self.groupBoxModulationAccuracy.setObjectName("groupBoxModulationAccuracy")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(self.groupBoxModulationAccuracy)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.radioButtonLowModulationAccuracy = QtWidgets.QRadioButton(self.groupBoxModulationAccuracy)
        self.radioButtonLowModulationAccuracy.setObjectName("radioButtonLowModulationAccuracy")
        self.verticalLayout_7.addWidget(self.radioButtonLowModulationAccuracy)
        self.radioButtonMediumModulationAccuracy = QtWidgets.QRadioButton(self.groupBoxModulationAccuracy)
        self.radioButtonMediumModulationAccuracy.setObjectName("radioButtonMediumModulationAccuracy")
        self.verticalLayout_7.addWidget(self.radioButtonMediumModulationAccuracy)
        self.radioButtonHighModulationAccuracy = QtWidgets.QRadioButton(self.groupBoxModulationAccuracy)
        self.radioButtonHighModulationAccuracy.setObjectName("radioButtonHighModulationAccuracy")
        self.verticalLayout_7.addWidget(self.radioButtonHighModulationAccuracy)
        self.verticalLayout_9.addWidget(self.groupBoxModulationAccuracy)
        spacerItem = QtWidgets.QSpacerItem(20, 500, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_9.addItem(spacerItem)
        self.tabWidget.addTab(self.tabGeneration, "")
        self.tabView = QtWidgets.QWidget()
        self.tabView.setObjectName("tabView")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.tabView)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_7 = QtWidgets.QLabel(self.tabView)
        self.label_7.setObjectName("label_7")
        self.horizontalLayout_2.addWidget(self.label_7)
        self.comboBoxDefaultView = QtWidgets.QComboBox(self.tabView)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.comboBoxDefaultView.sizePolicy().hasHeightForWidth())
        self.comboBoxDefaultView.setSizePolicy(sizePolicy)
        self.comboBoxDefaultView.setObjectName("comboBoxDefaultView")
        self.comboBoxDefaultView.addItem("")
        self.comboBoxDefaultView.addItem("")
        self.comboBoxDefaultView.addItem("")
        self.horizontalLayout_2.addWidget(self.comboBoxDefaultView)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.checkBoxShowConfirmCloseDialog = QtWidgets.QCheckBox(self.tabView)
        self.checkBoxShowConfirmCloseDialog.setObjectName("checkBoxShowConfirmCloseDialog")
        self.verticalLayout.addWidget(self.checkBoxShowConfirmCloseDialog)
        self.checkBoxHoldShiftToDrag = QtWidgets.QCheckBox(self.tabView)
        self.checkBoxHoldShiftToDrag.setObjectName("checkBoxHoldShiftToDrag")
        self.verticalLayout.addWidget(self.checkBoxHoldShiftToDrag)
        self.checkBoxPauseTime = QtWidgets.QCheckBox(self.tabView)
        self.checkBoxPauseTime.setObjectName("checkBoxPauseTime")
        self.verticalLayout.addWidget(self.checkBoxPauseTime)
        self.checkBoxAlignLabels = QtWidgets.QCheckBox(self.tabView)
        self.checkBoxAlignLabels.setObjectName("checkBoxAlignLabels")
        self.verticalLayout.addWidget(self.checkBoxAlignLabels)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.labelFontSize = QtWidgets.QLabel(self.tabView)
        self.labelFontSize.setObjectName("labelFontSize")
        self.horizontalLayout.addWidget(self.labelFontSize)
        self.doubleSpinBoxFontSize = QtWidgets.QDoubleSpinBox(self.tabView)
        self.doubleSpinBoxFontSize.setMinimum(1.0)
        self.doubleSpinBoxFontSize.setObjectName("doubleSpinBoxFontSize")
        self.horizontalLayout.addWidget(self.doubleSpinBoxFontSize)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_9 = QtWidgets.QLabel(self.tabView)
        self.label_9.setObjectName("label_9")
        self.horizontalLayout_4.addWidget(self.label_9)
        self.comboBoxTheme = QtWidgets.QComboBox(self.tabView)
        self.comboBoxTheme.setObjectName("comboBoxTheme")
        self.comboBoxTheme.addItem("")
        self.comboBoxTheme.addItem("")
        self.comboBoxTheme.addItem("")
        self.horizontalLayout_4.addWidget(self.comboBoxTheme)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.labelIconTheme = QtWidgets.QLabel(self.tabView)
        self.labelIconTheme.setObjectName("labelIconTheme")
        self.horizontalLayout_5.addWidget(self.labelIconTheme)
        self.comboBoxIconTheme = QtWidgets.QComboBox(self.tabView)
        self.comboBoxIconTheme.setObjectName("comboBoxIconTheme")
        self.comboBoxIconTheme.addItem("")
        self.comboBoxIconTheme.addItem("")
        self.horizontalLayout_5.addWidget(self.comboBoxIconTheme)
        self.verticalLayout.addLayout(self.horizontalLayout_5)
        self.groupBoxSpectrogramColormap = QtWidgets.QGroupBox(self.tabView)
        self.groupBoxSpectrogramColormap.setObjectName("groupBoxSpectrogramColormap")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.groupBoxSpectrogramColormap)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.scrollAreaSpectrogramColormap = QtWidgets.QScrollArea(self.groupBoxSpectrogramColormap)
        self.scrollAreaSpectrogramColormap.setWidgetResizable(True)
        self.scrollAreaSpectrogramColormap.setObjectName("scrollAreaSpectrogramColormap")
        self.scrollAreaWidgetSpectrogramColormapContents = QtWidgets.QWidget()
        self.scrollAreaWidgetSpectrogramColormapContents.setGeometry(QtCore.QRect(0, 0, 762, 466))
        self.scrollAreaWidgetSpectrogramColormapContents.setObjectName("scrollAreaWidgetSpectrogramColormapContents")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.scrollAreaWidgetSpectrogramColormapContents)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.scrollAreaSpectrogramColormap.setWidget(self.scrollAreaWidgetSpectrogramColormapContents)
        self.verticalLayout_2.addWidget(self.scrollAreaSpectrogramColormap)
        self.verticalLayout.addWidget(self.groupBoxSpectrogramColormap)
        self.tabWidget.addTab(self.tabView, "")
        self.tabFieldtypes = QtWidgets.QWidget()
        self.tabFieldtypes.setObjectName("tabFieldtypes")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.tabFieldtypes)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.tblLabeltypes = QtWidgets.QTableView(self.tabFieldtypes)
        self.tblLabeltypes.setAlternatingRowColors(True)
        self.tblLabeltypes.setObjectName("tblLabeltypes")
        self.horizontalLayout_3.addWidget(self.tblLabeltypes)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.btnAddLabelType = QtWidgets.QToolButton(self.tabFieldtypes)
        icon = QtGui.QIcon.fromTheme("list-add")
        self.btnAddLabelType.setIcon(icon)
        self.btnAddLabelType.setObjectName("btnAddLabelType")
        self.verticalLayout_3.addWidget(self.btnAddLabelType)
        self.btnRemoveLabeltype = QtWidgets.QToolButton(self.tabFieldtypes)
        icon = QtGui.QIcon.fromTheme("list-remove")
        self.btnRemoveLabeltype.setIcon(icon)
        self.btnRemoveLabeltype.setObjectName("btnRemoveLabeltype")
        self.verticalLayout_3.addWidget(self.btnRemoveLabeltype)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem1)
        self.horizontalLayout_3.addLayout(self.verticalLayout_3)
        self.verticalLayout_5.addLayout(self.horizontalLayout_3)
        spacerItem2 = QtWidgets.QSpacerItem(20, 203, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_5.addItem(spacerItem2)
        self.tabWidget.addTab(self.tabFieldtypes, "")
        self.tab_plugins = QtWidgets.QWidget()
        self.tab_plugins.setObjectName("tab_plugins")
        self.tabWidget.addTab(self.tab_plugins, "")
        self.tabDevices = QtWidgets.QWidget()
        self.tabDevices.setObjectName("tabDevices")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.tabDevices)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.labelInfoDeviceTable = QtWidgets.QLabel(self.tabDevices)
        font = QtGui.QFont()
        font.setItalic(True)
        self.labelInfoDeviceTable.setFont(font)
        self.labelInfoDeviceTable.setToolTip("")
        self.labelInfoDeviceTable.setWordWrap(True)
        self.labelInfoDeviceTable.setObjectName("labelInfoDeviceTable")
        self.verticalLayout_8.addWidget(self.labelInfoDeviceTable)
        self.tblDevices = QtWidgets.QTableView(self.tabDevices)
        self.tblDevices.setAlternatingRowColors(True)
        self.tblDevices.setShowGrid(False)
        self.tblDevices.setObjectName("tblDevices")
        self.tblDevices.horizontalHeader().setDefaultSectionSize(200)
        self.tblDevices.verticalHeader().setVisible(False)
        self.verticalLayout_8.addWidget(self.tblDevices)
        self.labelDeviceMissingInfo = QtWidgets.QLabel(self.tabDevices)
        font = QtGui.QFont()
        font.setItalic(False)
        self.labelDeviceMissingInfo.setFont(font)
        self.labelDeviceMissingInfo.setWordWrap(True)
        self.labelDeviceMissingInfo.setObjectName("labelDeviceMissingInfo")
        self.verticalLayout_8.addWidget(self.labelDeviceMissingInfo)
        self.line = QtWidgets.QFrame(self.tabDevices)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout_8.addWidget(self.line)
        self.groupBoxNativeOptions = QtWidgets.QGroupBox(self.tabDevices)
        self.groupBoxNativeOptions.setObjectName("groupBoxNativeOptions")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.groupBoxNativeOptions)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.labelLibDirs = QtWidgets.QLabel(self.groupBoxNativeOptions)
        self.labelLibDirs.setObjectName("labelLibDirs")
        self.gridLayout_5.addWidget(self.labelLibDirs, 2, 0, 1, 1)
        self.lineEditIncludeDirs = QtWidgets.QLineEdit(self.groupBoxNativeOptions)
        self.lineEditIncludeDirs.setObjectName("lineEditIncludeDirs")
        self.gridLayout_5.addWidget(self.lineEditIncludeDirs, 3, 2, 1, 1)
        self.labelNativeRebuildInfo = QtWidgets.QLabel(self.groupBoxNativeOptions)
        self.labelNativeRebuildInfo.setWordWrap(True)
        self.labelNativeRebuildInfo.setObjectName("labelNativeRebuildInfo")
        self.gridLayout_5.addWidget(self.labelNativeRebuildInfo, 1, 0, 1, 3)
        self.lineEditLibDirs = QtWidgets.QLineEdit(self.groupBoxNativeOptions)
        self.lineEditLibDirs.setObjectName("lineEditLibDirs")
        self.gridLayout_5.addWidget(self.lineEditLibDirs, 2, 2, 1, 1)
        self.labelIncludeDirs = QtWidgets.QLabel(self.groupBoxNativeOptions)
        self.labelIncludeDirs.setObjectName("labelIncludeDirs")
        self.gridLayout_5.addWidget(self.labelIncludeDirs, 3, 0, 1, 1)
        self.btnViewBuildLog = QtWidgets.QPushButton(self.groupBoxNativeOptions)
        icon = QtGui.QIcon.fromTheme("utilities-log-viewer")
        self.btnViewBuildLog.setIcon(icon)
        self.btnViewBuildLog.setObjectName("btnViewBuildLog")
        self.gridLayout_5.addWidget(self.btnViewBuildLog, 4, 1, 1, 1)
        self.btnRebuildNative = QtWidgets.QPushButton(self.groupBoxNativeOptions)
        self.btnRebuildNative.setEnabled(True)
        icon = QtGui.QIcon.fromTheme("view-refresh")
        self.btnRebuildNative.setIcon(icon)
        self.btnRebuildNative.setObjectName("btnRebuildNative")
        self.gridLayout_5.addWidget(self.btnRebuildNative, 4, 0, 1, 1)
        self.labelRebuildNativeStatus = QtWidgets.QLabel(self.groupBoxNativeOptions)
        self.labelRebuildNativeStatus.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.labelRebuildNativeStatus.setObjectName("labelRebuildNativeStatus")
        self.gridLayout_5.addWidget(self.labelRebuildNativeStatus, 4, 2, 1, 1)
        self.verticalLayout_8.addWidget(self.groupBoxNativeOptions)
        self.line_2 = QtWidgets.QFrame(self.tabDevices)
        self.line_2.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.verticalLayout_8.addWidget(self.line_2)
        self.groupBox_3 = QtWidgets.QGroupBox(self.tabDevices)
        self.groupBox_3.setObjectName("groupBox_3")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.groupBox_3)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.label = QtWidgets.QLabel(self.groupBox_3)
        self.label.setObjectName("label")
        self.gridLayout_2.addWidget(self.label, 0, 0, 1, 1)
        self.lineEditGRPythonInterpreter = QtWidgets.QLineEdit(self.groupBox_3)
        self.lineEditGRPythonInterpreter.setObjectName("lineEditGRPythonInterpreter")
        self.gridLayout_2.addWidget(self.lineEditGRPythonInterpreter, 0, 1, 1, 1)
        self.btnChooseGRPythonInterpreter = QtWidgets.QToolButton(self.groupBox_3)
        self.btnChooseGRPythonInterpreter.setObjectName("btnChooseGRPythonInterpreter")
        self.gridLayout_2.addWidget(self.btnChooseGRPythonInterpreter, 0, 2, 1, 1)
        self.verticalLayout_8.addWidget(self.groupBox_3)
        self.line_3 = QtWidgets.QFrame(self.tabDevices)
        self.line_3.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_3.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_3.setObjectName("line_3")
        self.verticalLayout_8.addWidget(self.line_3)
        self.gridLayout_3 = QtWidgets.QGridLayout()
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.label_8 = QtWidgets.QLabel(self.tabDevices)
        self.label_8.setObjectName("label_8")
        self.gridLayout_3.addWidget(self.label_8, 0, 0, 1, 1)
        self.spinBoxNumSendingRepeats = QtWidgets.QSpinBox(self.tabDevices)
        self.spinBoxNumSendingRepeats.setProperty("showGroupSeparator", False)
        self.spinBoxNumSendingRepeats.setMaximum(999999999)
        self.spinBoxNumSendingRepeats.setDisplayIntegerBase(10)
        self.spinBoxNumSendingRepeats.setObjectName("spinBoxNumSendingRepeats")
        self.gridLayout_3.addWidget(self.spinBoxNumSendingRepeats, 0, 1, 1, 1)
        self.label_5 = QtWidgets.QLabel(self.tabDevices)
        self.label_5.setObjectName("label_5")
        self.gridLayout_3.addWidget(self.label_5, 1, 0, 1, 1)
        self.doubleSpinBoxRAMThreshold = QtWidgets.QDoubleSpinBox(self.tabDevices)
        self.doubleSpinBoxRAMThreshold.setMinimum(1.0)
        self.doubleSpinBoxRAMThreshold.setMaximum(100.0)
        self.doubleSpinBoxRAMThreshold.setObjectName("doubleSpinBoxRAMThreshold")
        self.gridLayout_3.addWidget(self.doubleSpinBoxRAMThreshold, 1, 1, 1, 1)
        self.verticalLayout_8.addLayout(self.gridLayout_3)
        self.tabWidget.addTab(self.tabDevices, "")
        self.verticalLayout_6.addWidget(self.tabWidget)

        self.retranslateUi(DialogOptions)
        self.tabWidget.setCurrentIndex(4)

    def retranslateUi(self, DialogOptions):
        _translate = QtCore.QCoreApplication.translate
        DialogOptions.setWindowTitle(_translate("DialogOptions", "Options"))
        self.labelFuzzingSamples.setText(_translate("DialogOptions", "Samples"))
        self.checkBoxDefaultFuzzingPause.setToolTip(_translate("DialogOptions", "<html><head/><body><p>If you disable the default pause, the pause of the fuzzed message will be used.</p></body></html>"))
        self.checkBoxDefaultFuzzingPause.setText(_translate("DialogOptions", "Use a default pause for fuzzed messages"))
        self.checkBoxMultipleModulations.setText(_translate("DialogOptions", "Enable modulation profiles"))
        self.groupBoxModulationAccuracy.setTitle(_translate("DialogOptions", "Modulation Accuracy"))
        self.radioButtonLowModulationAccuracy.setText(_translate("DialogOptions", "Low (2x8 bit) - Recommended for HackRF and RTL-SDR"))
        self.radioButtonMediumModulationAccuracy.setText(_translate("DialogOptions", "Medium (2x16 bit) - Recommended for BladeRF, PlutoSDR and SDRPlay"))
        self.radioButtonHighModulationAccuracy.setText(_translate("DialogOptions", "High (2x32 bit) - Recommended if you are not sure what to choose"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tabGeneration), _translate("DialogOptions", "Generation"))
        self.label_7.setText(_translate("DialogOptions", "Default View:"))
        self.comboBoxDefaultView.setItemText(0, _translate("DialogOptions", "Bit"))
        self.comboBoxDefaultView.setItemText(1, _translate("DialogOptions", "Hex"))
        self.comboBoxDefaultView.setItemText(2, _translate("DialogOptions", "ASCII"))
        self.checkBoxShowConfirmCloseDialog.setText(_translate("DialogOptions", "Show \"confirm close\" dialog"))
        self.checkBoxHoldShiftToDrag.setToolTip(_translate("DialogOptions", "<html><head/><body><p>If checked, you need to <span style=\" font-weight:600;\">hold the Shift key to drag</span> with the mouse inside graphic views like the drawn signal in Interpretation tab, while making a selection with the mouse does not require holding any buttons.</p><p>If unchecked, this is inverted: Hold shift to make a selection, and drag by default.</p></body></html>"))
        self.checkBoxHoldShiftToDrag.setText(_translate("DialogOptions", "Hold shift to drag"))
        self.checkBoxPauseTime.setText(_translate("DialogOptions", "Show pauses as time"))
        self.checkBoxAlignLabels.setText(_translate("DialogOptions", "Align on labels"))
        self.labelFontSize.setText(_translate("DialogOptions", "<html><head/><body><p>Application font size (<span style=\" font-weight:600;\">restart</span> for full effect):</p></body></html>"))
        self.doubleSpinBoxFontSize.setSuffix(_translate("DialogOptions", "pt"))
        self.label_9.setText(_translate("DialogOptions", "Choose application theme (requires restart):"))
        self.comboBoxTheme.setItemText(0, _translate("DialogOptions", "native look (default)"))
        self.comboBoxTheme.setItemText(1, _translate("DialogOptions", "fallback theme"))
        self.comboBoxTheme.setItemText(2, _translate("DialogOptions", "fallback theme (dark)"))
        self.labelIconTheme.setText(_translate("DialogOptions", "Choose icon theme (requires restart):"))
        self.comboBoxIconTheme.setItemText(0, _translate("DialogOptions", "bundled icons (default)"))
        self.comboBoxIconTheme.setItemText(1, _translate("DialogOptions", "native icon theme"))
        self.groupBoxSpectrogramColormap.setTitle(_translate("DialogOptions", "Spectrogram Colormap"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tabView), _translate("DialogOptions", "View"))
        self.btnAddLabelType.setText(_translate("DialogOptions", "..."))
        self.btnRemoveLabeltype.setText(_translate("DialogOptions", "..."))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tabFieldtypes), _translate("DialogOptions", "Fieldtypes"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_plugins), _translate("DialogOptions", "Plugins"))
        self.labelInfoDeviceTable.setText(_translate("DialogOptions", "<html><head/><body><p>Use the checkboxes in the table below to choose device backends and enable or disable devices. Disabled devices will not show up in device related dialogs such as send or receive.</p></body></html>"))
        self.labelDeviceMissingInfo.setText(_translate("DialogOptions", "<html><head/><body><p>Missing a native backend? Perform a <a href=\"health_check\"><span style=\" text-decoration: underline; color:#0000ff;\">health check</span></a>! If GNU Radio backend is not available double check the GNU Radio settings below.</p></body></html>"))
        self.groupBoxNativeOptions.setTitle(_translate("DialogOptions", "Native options"))
        self.labelLibDirs.setText(_translate("DialogOptions", "Library directories:"))
        self.lineEditIncludeDirs.setPlaceholderText(_translate("DialogOptions", "Comma separated list of additional include directories"))
        self.labelNativeRebuildInfo.setText(_translate("DialogOptions", "You can rebuild the native device extensions here. This is useful, when you installed a device driver afterwards or your drivers are stored in an unusual location."))
        self.lineEditLibDirs.setPlaceholderText(_translate("DialogOptions", "Comma separated list of additional library directories"))
        self.labelIncludeDirs.setText(_translate("DialogOptions", "Include directories:"))
        self.btnViewBuildLog.setText(_translate("DialogOptions", "View log"))
        self.btnRebuildNative.setToolTip(_translate("DialogOptions", "<html><head/><body><p>Rebuild the native device extensions. You need to restart URH after this, to use new extensions.</p></body></html>"))
        self.btnRebuildNative.setText(_translate("DialogOptions", "Rebuild"))
        self.labelRebuildNativeStatus.setText(_translate("DialogOptions", "Rebuild <x> new device extensions. Please restart URH to use them."))
        self.groupBox_3.setTitle(_translate("DialogOptions", "GNU Radio options (optional)"))
        self.label.setToolTip(_translate("DialogOptions", "<html><head/><body><p>Choose a python interpreter which has access to your GNU Radio installation, that is, you can do <span style=\" font-weight:600;\">python -c &quot;import gnuradio&quot;</span> with it on the command line.</p></body></html>"))
        self.label.setText(_translate("DialogOptions", "Python Interpreter:"))
        self.lineEditGRPythonInterpreter.setToolTip(_translate("DialogOptions", "<html><head/><body><p>Choose a python interpreter which has access to your GNU Radio installation, that is, you can do <span style=\" font-weight:600;\">python -c &quot;import gnuradio&quot;</span> with it on the command line.</p></body></html>"))
        self.lineEditGRPythonInterpreter.setPlaceholderText(_translate("DialogOptions", "Enter python interpreter path e.g. /usr/bin/python"))
        self.btnChooseGRPythonInterpreter.setText(_translate("DialogOptions", "..."))
        self.label_8.setText(_translate("DialogOptions", "Default sending repititions:"))
        self.spinBoxNumSendingRepeats.setSpecialValueText(_translate("DialogOptions", "Infinite"))
        self.label_5.setText(_translate("DialogOptions", "Use this percentage of available RAM for buffer allocation:"))
        self.doubleSpinBoxRAMThreshold.setSuffix(_translate("DialogOptions", "%"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tabDevices), _translate("DialogOptions", "Device"))
from urh.ui.KillerDoubleSpinBox import KillerDoubleSpinBox
